/***********************************************************************************************
*   delay_processing.ASM - 'Ping-Pong' stereo delay effects algorithm with cross-feedback      *
*                                                                                              *
*                                                                                              *
*   For use with the ADSP-21262 EZ-KIT Evaluation Board                                        *
*   Based on Fig.8.4.1 of Introduction to Signal Processing.                                   *
*   By S. J. Orfanidis - 1996, Prentice-Hall                                                   *
*   ISBN 0-13-209172-0                                                                         *
*                                                                                              *
*   What the stereo delay effect does?                                                         *
*   Creating multiple delay typte offects can be obtained from implementing                    *
*   simple low-order FIR or IIR filters.  Many DSP audio effects processors widely use         *
*   variations of this effect created with multiple delay lines.                               *
*                                                                                              *
*   Stereo delay effects can be implemented by coupling the left and right channels.           *
*   The left and right channels are coupled by introducing cross-feedback coefficients, so     *
*   that the delayed output of one channel is fed into the input of the other channel.         *
*                                                                                              *
*   Assuming the feedback filters are plain multipliers, GL(z)=aL, GR(z)=aR,                   *
*   the sample processing algorithm for each pair xL,xR do:                                    *
*                                                                                              *
*           sampleLL = tap(L, wL, pL, L)              L-th tap of left delay-line              *
*           sampleRR = tap(R, wR, pR, R)              R-th tap of right delay-line             *
*           yL = cL * xL + sLL                        left output filter                       *
*           yR = cR * xR + sRR                        right output filter                      *
*           *pL = bL * xL + aL * sLL + dR * sRR       input to left delay                      *
*           *pR = bR * xR + aR * sRR + dL * sLL       input to right delay                     *
*           update_left_delay(L, wL, &pL)             update left delay                        *
*           update_right_delay(R, wR, &pR)            update right delay                       *
*                                                                                              *
*   The I/O equations in the z-domain are:                                                     *
*                                                                                              *
*                  YL(z) = HLL(z)*XL(x) + HLR(z)*XR(z)                                         *
*                  YR(z) = HRL(z)*XL(z) + HRR(z)*XR(z)                                         *
*                                                                                              *
*                                                                                              *
*       xL(n) ------------------------------------------|>------>O--------> yL(n)              *
*               |                                       cL     + ^                             *
*               |                    ________                    |                             *
*               |                   |   aL   |                   |                             *
*               |        |----------| G_L(z) |---------|         |                             *
*               |        |          |________|         |         |                             *    
*		        |        |                             |         |                             *
*               |        |       _______________       |         |                             *
*               |        |      |               |      |         |                             *
*               |        v +    |               |      |         |                             *
*               |---|>-->O----->|     Z^(-L)    |----------------|                             *
*                   bL   ^ +    |               |         |                                    *
*                        |      |_______________|         |                                    *
*                        |                                |                                    *
*                        |                                |                                    *
*                        |                          dL    |                                    *
*                        |------------\   /---------<|----|                                    *
*                                      \ /                                                     *
*                                       X                                                      *
*                                      / \          dR                                         *
*                        |------------/   \---------<|----|                                    *
*                        |                                |                                    *
*                        |                                |                                    *
*                        |       _______________          |                                    *
*                        |      |               |         |                                    *
*                   bR   v +    |               |         |                                    *
*               |---|>-->O----->|     Z^(-R)    |----------------|                             *
*               |        ^ +    |               |      |         |                             *
*               |        |      |_______________|      |         |                             *
*   		    |        |                             |         |                             *
*               |        |           ________          |         |                             *
*               |        |          |   aR   |         |         |                             *
*               |        |----------| G_R(z) |---------|         |                             *
*               |                   |________|                   |                             *                
*               |                                                |                             *
*               |                                              + v                             *                             *
*       xR(n) ------------------------------------------|>------>O--------> yR(n)              *
*                                                      cL                                      *
*                                                                                              *
*                                                                                              *
************************************************************************************************/

#include <def21262.h>

.extern  Left_channelIn, Right_channelIn,Left_channelOut, Right_channelOut;
.global  delay_audio;
.global	 Init_Delay_Buffers;
.global	 change_delay_time;
.global	 select_feedback_gain;


/* ----------- INTERRUPT/FLAG REVERB FX DEMO CONTROL PARAMETERS ---------*/
.section /dm	seg_dmda;


/* ------------ Digital Ping Pong Delay Parameters ----------------------*/


/* 32 bit filter coefficients are in floating point format */

#define aL  	0.0   			    //aL = 0 - left self-feedback
#define aR  	0.0          	    //aR = 0 - right self-feedback

.var    bL  =   0.8;          	    //bL = 0.8 - left delay gain
.var    bR  =   0.8;        	    //bR = 0.8 - right delay gain
 
#define cL  	0.5       	        //cL = 0.5 - left direct path gain
#define cR  	0.5        	        //cR = 0.5 - right direct path gain

#define dL  	0.5      	        //dL = 0.5 - cross feedback from L to R
#define dR  	0.5          	    //dR = 0.5 - cross feedback from R to L


/* ---- Stereo Reflection Settings for a Slow Decay Delay Processor  ----- */

#define L_2   	24001               //TL = L/fs = 24000/48000 = 0.5 sec, maximum Left Channel buffer length = L + 1 
#define R_2   	24001              	//TR = R/fs = 24000/48000 = 0.5 sec, maximum Right Channel buffer length in 32-bit words

#define L_2E    L_2*4               // maximum LC external buffer length for 8-bit memory 
#define R_2E    R_2*4               // maximum RC external buffer length for 8-bit memory 

.var	L_delay_time = 24001*4; 	// controls length of (L2 register) left delay buffer 
									// length value always << max buffer length! 
.var	R_delay_time = 24001*4; 	// controls length of (L3 register) right delay buffer 
								    // length value always << max buffer length! 

.var	IRQ1_counter = 0x00000004;	/* selects preset 1 on first IRQ1 assertion */
.var	IRQ2_counter = 0x00000004;	/* selects preset 1 on first IRQ2 assertion */
								    								    
.var count2;								    



/* ------------- DATA MEMORY FILTER BUFFERS ---------------------------------*/
.section /dm/ZERO_INIT/DMAONLY	dm_delay;   // note to chage the ldf file for this

.var	wL[L_2E ];             		// left delay-line buffer 
.var	wR[R_2E ];             		// right delay-line buffer 


/* ---------------------- PROGRAM MEMORY CODE ---------------------------------*/


.section/pm seg_pmco;

Init_Delay_Buffers:
BIT SET MODE1 CBUFEN;              // Enable Circular buffer

NOP;

B2 = wL; L2 = L_2E;                /* delay-line buffer pointer and length */
B3 = wR; L3 = R_2E;                /* delay-line buffer pointer and length */
m2 = 1; m3 = 1;	
   	
RTS;
Init_Delay_Buffers.end:


delay_audio:

/* ---------- get input samples from data holders ---------- */
	r2 =  -31;    
    r0 = dm(Left_channelIn);     // left channel input sample 
//	r1 = dm(Right_channelIn);    // right channel input sample 
	r1 = 0x00000000;		     // clear out right channel for now. For true stereo delay, remove this instruction

	f0 = float r0 by r2;
	f1 = float r1 by r2; 
	
	
/* ---------- tap outputs of circular delay lines ---------- */

    L2 = dm(L_delay_time);			// get current length of left delay line 
	r5 = L2;
	r7 = 0x4;                            
	r5 = r5 - r7;					// subtract value of 4 to get oldest sample for 8-bit memory(Left channel) 
	L3 = dm(R_delay_time);			// get current length of right delay line 
	r6 = L3;
	r6 = r6 - r7;					// subtract value of 4 to get oldest sample for 8-bit memory(Right channel) 

	ustat3 = PPBHC| PPDUR3;         // Initially disable parallel port
	dm(PPCTL) = ustat3; 
   	
/* --------- r3 = sLL = L-th tap of left delay ---------  */
   	m2 =  r5;  modify(i2, m2);      // point to d-th tap of left delay line 
	r2=i2;  dm(EIPP)=r2;            // external index register of parrallel port pointing to d-th tap
   	r2 = 1; dm(EMPP) = r2;          // modify register
   	r2 = 4; dm(ECPP) = r2;          // external memory count register for one value
   	
   	ustat3 = PPEN| PPBHC| PPDUR4;
   	
   	dm(PPCTL) = ustat3;             // Enable parallel port
   	
   	r3 = dm(RXPP);                  // get the dth tap value of left channel
   	
/*************************************************************************************************/   	
/* Known duration access ofparallel port requires 15 cycles                                      */
/* 19 cycles = (ALE cycles(3 cycles) + No of 8-bit words(4) * duration for each word (4 cycles)) */
/*************************************************************************************************/    
    
   	r5 = -r5;                
   	m2 = r5;  modify(i2,m2);        // point back to 0-th tap of left delay buffer
   	
    m3 =  r6;  modify(i3, m3);      // point to d-th tap of right delay line 
   	 
   	nop;nop;                        // Nops are used to ensure that the 15 cycles    
    nop;nop;                        // required for 32-bit word transfer are utilized
    nop;nop;                        // In place of these nops the core could do any kind of processing
    nop;nop;
    nop;nop;
    nop;nop;
    nop;nop;
    

   	ustat3 = PPBHC| PPDUR3;         //disables parallel port
   	dm(PPCTL) = ustat3; 
   	
   
    	
/* ---------  r4 = sRR = R-th tap of right delay ---------------- */
   	
  	r2=i3;  dm(EIPP)=r2;            // index register of parrallel port pointing to d-th tap 
   	r2 = 1; dm(EMPP) = r2;          // modify register
   	r2 = 4; dm(ECPP) = r2;          // count register for one value
   	
   	ustat3 = PPEN| PPBHC| PPDUR4;
   	
   	dm(PPCTL) = ustat3;             // Enable parallel port
   	
   	r4 = dm(RXPP);                  // get the dth tap value of right channel

/*************************************************************************************************/   	   	   	
/* Known duration access ofparallel port requires 15 cycles                                      */
/* 19 cycles = (ALE cycles(3 cycles) + No of 8-bit words(4) * duration for each word (4 cycles)) */
/*************************************************************************************************/   	
   	
   	r6 = -r6;
   	m3 = r6;  modify(i3,m3);     		// point back to 0-th tap of right delay buffer
   	
   	r2 = -31;
   	f3 = float r3 by r2;  	
    
    f11 = cL;
   	f11 = f0 * f11;                     // fll = cL * xL
   	f10 = f3 + f11;                     // f10 = cL * xL + sLL = left output 
   	
   	r2 = 31;
  	r10 = fix f10 by r2; 
  	
  	
   	
   	dm(Left_channelOut) = r10;          // send result to left output channel
   	
   	nop; nop;                           // nops required for known duration access
    nop; nop; 
    nop; nop;
    nop; nop; 
   	   
   	
   	ustat3 = PPBHC| PPDUR3;             // disables parallel port
   	dm(PPCTL) = ustat3;
   	

   	r2 = -31;
   	f4 = float r4 by r2;
   	
   
   	f11 = dm(bL);                       
   	f12 = f0* f11; 						// f12= xL * bL
    f11 = aL;							
   	f11 = f11 * f3;						// f11= aL * sLL
   	f12 = f11 + f12;                    // f12= aL * sLL + xL * bL
   	f11 = dR;
   	f11 = f11* f4;                      // f11= dR * sRR
   	f12 = f11 + f12;					// f12= dR * sRR + aL * sLL + xL * bL
   	
   	r2 = 31;
  	r12 = fix f12 by r2; 
  

    	
/* ---------- put value from r12 result into tap-0 of left delay-line ----------- */    
   	 

    r2=i2;  dm(EIPP)=r2;              // index register of parrallel port pointing to 0-th tap
   	r2 = 1; dm(EMPP) = r2;            // modify register
   	r2 = 4; dm(ECPP) = r2;            // count register for one value
   	
   	ustat3 = PPEN| PPBHC| PPDUR4|PPTRAN;
   	
   	dm(PPCTL) = ustat3;               // Enable parallel port in transmit mode
   	
   	dm(TXPP)  = r12;
   	 
   	f11 = cR;
	f11 = f1*f11;                     // f11 = xR * cR
	f10= f11+f4;                      // f10 = xR * cR  + sRR = right output
	r2 = 31;
	r10 = fix f10 by r2; 
	
   	dm(Right_channelOut) = r10;       // send result to right output channel
   	
   	
    f11 = dm(bR); 
  	f12 = f11 * f1;                   // f12 = bR * xR;
   	f11 = aR;          
   	f11 = f11 * f4;                   // f11 = aR * sRR
   	f12 = f12 + f11;                  // f12 = bR * xR + aR * sRR
   	f11 = dL;
   	f11 = f11* f3;                    // f11 = dL * sLL
   	f12 = f12 + f11;                  // f12 = bR * xR + aR * sRR + dL * sLL   	
   	nop; nop; nop; nop; 
   	
/* -----------  backshift pointer, update left circular delay-line buffer ----------*/                   
  	m2 = -4;  modify(i2, m2); 
   	
   	ustat4 = PPDUR3|PPTRAN;           // disables parallel port
   	dm(PPCTL) = ustat4;
   	
   	r2 = 31;
   	r12 = fix f12 by r2; 
 
   	
   	
/* ------  put value from MAC into delay line into tap-0 of right delay-line -------- */
      	
     r2= i3;  dm(EIPP)=r2;  		  // index register of parrallel port pointing to 0-th tap
   	 r2 = 1; dm(EMPP) = r2;           // modify register
   	 r2 = 4; dm(ECPP) = r2;           // count register for one value
   	 
   	ustat3 = PPEN| PPBHC| PPDUR4|PPTRAN;
   	
   	dm(PPCTL) = ustat3;               // Enable parallel port in transmit mode
   	
   	dm(TXPP)  = r12; 
   	
   	
   	nop; nop; nop;                    // nops for known duration access using parallel port
   	nop; nop; nop;
   	nop; nop; nop;
   	nop; nop; nop;
   	nop; nop; nop; 
   	nop; nop;
    	
/* ----------  backshift pointer, update right circular delay-line buffer -------- */                   
  	m3 = -4;  modify(i3, m3);
   	
   	ustat4 = PPDUR3|PPTRAN;
   	dm(PPCTL) = ustat4;
   	
   	r0 = dm(count2);
   	r0 = r0 +1;
   	
   	dm(count2) =  r0; 
   	
   	rts;

delay_audio.end:


/* ------------------------------------------------------------------------------------ */
/*                                                                                      */
/*                        IRQ2 Pushbutton Interrupt Service Routine                     */
/*                                                                                      */
/*      This routine allows the user to modify reverb presets on-the-fly to allow for   */
/*      emulation of either large 'valley' echos, medium hallway echos, or a small      */
/*      enclosed room with hard surfaces.                                               */
/*                                                                                      */
/*      Default before 1st IRQ push:	Delay = 20.83 msec                              */
/*      1st Pushbutton Press:	Delay Setting #1 -   35 msec                            */
/*      2nd Pushbutton Press:	Delay Setting #2 - 62.5 msec                            */
/*      3rd Pushbutton Press:	Delay Setting #3 -  125 msec                            */
/*      4th Pushbutton Press:	Delay Setting #4 - 0.25 sec                             */
/*      5th Pushbutton Press:	Delay Setting #4 - 0.50 sec                             */
/*      6th Pushbutton Press:	Delay Setting #4 -  1.0 sec                             */
/*      7th Pushbutton Press:	Reverts back to 1st Pushbutton Press                    */
/*                                                                                      */
/*      The pushbutton setting is shown by the active LED setting, all others are off	*/	
/* ------------------------------------------------------------------------------------ */
						  
change_delay_time:
	bit set mode1 SRRFH;				/* enable background register file */
	NOP; 								/* 1 CYCLE LATENCY FOR WRITING TO MODE1 REGISER!!   */

	r13 = 6;							/* number of presets */	
	r15 = DM(IRQ1_counter);				/* get preset count */
	r15 = r15 + 1;						/* increment preset */
	comp (r15, r13);
	if ge r15 = r15 - r15;				/* reset to zero */
	DM(IRQ1_counter) = r15;				/* save preset count */

	r8 = pass r15;						/* get preset mode */
	if eq jump delay_settings_2;		/* check for count == 0 */
	r8 = r8 - 1;
	if eq jump delay_settings_3; 		/* check for count == 1 */
	r8 = r8 - 1;
	if eq jump delay_settings_4;		/* check for count == 3 */
	r8 = r8 - 1;
	if eq jump delay_settings_5;		/* check for count == 4 */
	r8 = r8 - 1;
	if eq jump delay_settings_6;		/* check for count == 4 */

delay_settings_1:						/* count therefore, is == 5 if you are here */
	/* Depth, or TD = D/fs = 1680/48000 = 35 msec   */
	r14 = 6724;		DM(L_delay_time) = r14;  //6724 = 1681*4 for parallel port access of  8-bit memory
	r14 = 6724;		DM(R_delay_time) = r14;

	
	jump done_changing_delay;

delay_settings_2:
	/* Depth, or TD = D/fs = 3000/48000 = 62.5 msec */  
	r14 = 12004;	  	DM(L_delay_time) = r14;  //12004 = 3001*4 for parallel port access of  8-bit memory
	r14 = 12004;	  	DM(R_delay_time) = r14;
	
		
	jump done_changing_delay;

delay_settings_3:
	/* Depth, or TD = D/fs = 6000/48000 = 125 msec  */			
	r14 = 24004;	  	DM(L_delay_time) = r14;  //24004 = 6001*4 for parallel port access of  8-bit memory
	r14 = 24004;	  	DM(R_delay_time) = r14;
	
	
	jump done_changing_delay;

delay_settings_4:
	/* Depth, or TD = D/fs = 12000/48000 = .25 sec  */			
	r14 = 48004;	  	DM(L_delay_time) = r14;  //48004 = 12001*4 for parallel port access of 8-bit memory
	r14 = 48004;	  	DM(R_delay_time) = r14;
	
	
	jump done_changing_delay;

delay_settings_5:
	/* Depth, or TD = D/fs = 18000/48000 = .375 sec  */			
	r14 = 72004;	  	DM(L_delay_time) = r14;  //72004 = 18001*4 for parallel port access of 8-bit memory
	r14 = 72004;	  	DM(R_delay_time) = r14;
	
	jump done_changing_delay;

delay_settings_6:
	/* Depth, or TD = D/fs = 24000/48000 = .50 sec  */			
	r14 = 96004;	  	DM(L_delay_time) = r14; //96004 = 24001*4 for parallel port access of 8-bit memory
	r14 = 96004;	  	DM(R_delay_time) = r14;
	
	
done_changing_delay:	
	rti(db);
	bit clr mode1 SRRFH;				/* switch back to primary register set */
	nop;

change_delay_time.end:
	
/* ------------------------------------------------------------------------------------ */
/*                                                                                      */
/*                     IRQ1 Pushbutton Interrupt Service Routine                        */
/*                                                                                      */
/*      This routine allows the user to modify delay feedback setting to allow for  	*/
/*      either a single reflection or multiple reflections                              */
/*                                                                                      */
/* ------------------------------------------------------------------------------------ */
						  
select_feedback_gain:
	bit set mode1 SRRFH;				/* enable background register file */
	NOP; 								/* 1 CYCLE LATENCY FOR WRITING TO MODE1 REGISER!!   */

	r13 = 6;							/* number of presets */	
	r15 = DM(IRQ2_counter);				/* get preset count */
	r15 = r15 + 1;						/* increment preset */
	comp (r15, r13);
	if ge r15 = r15 - r15;				/* reset to zero */
	DM(IRQ2_counter) = r15;				/* save preset count */

	r8 = pass r8;						/* get preset mode */
	if eq jump feedback_settings_2;		/* check for count == 0 */
	r8 = r8 - 1;
	if eq jump feedback_settings_3; 	/* check for count == 1 */
	r8 = r8 - 1;
	if eq jump feedback_settings_4;		/* check for count == 3 */
	r8 = r8 - 1;
	if eq jump feedback_settings_5;		/* check for count == 4 */
	r8 = r8 - 1;
	if eq jump feedback_settings_6;		/* check for count == 4 */

feedback_settings_1:					/* count therefore, is == 5 if you are here */
	f14 = 0.133333;	    DM(bL) = f14;
	r14 = 0.133333;	    DM(bR) = f14;
	
	jump exit_feedback_change;

feedback_settings_2:
	f14 = 0.266667;	    DM(bL) = f14;
	f14 = 0.266667;	    DM(bR) = f14;
			
	jump exit_feedback_change;

feedback_settings_3:
	f14 = 0.4;	        DM(bL) = f14;
	f14 = 0.4;	        DM(bR) = f14;
	
	jump exit_feedback_change;

feedback_settings_4:
	f14 = 0.533333;	    DM(bL) = f14;
	f14 = 0.533333;	    DM(bR) = f14;
	
	jump exit_feedback_change;

feedback_settings_5:
	f14 = 0.666667;	    DM(bL) = f14;
	f14 = 0.666667;	    DM(bR) = f14;
	
	jump exit_feedback_change;

feedback_settings_6:
	f14 = 0.8;	 		DM(bL) = f14;
	f14 = 0.8;			DM(bR) = f14;
	

exit_feedback_change:	
	rti(db);
	bit clr mode1 SRRFH;				/* switch back to primary register set */
	nop;

select_feedback_gain.end:
 



